/*
 * This part was hacked by Harald Kipp
 *
 * Bug reports should be sent to
 *
 *  harald@os2point.ping.de
 *  harald@haport.sesam.com
 *  Fido: 2:2448/434
 *
 * This module contains routines for stream socket connections.
 *
 */

#include <sys\types.h>
#include <netinet\in.h>
#include <sys\socket.h>
#include <netdb.h>
/*	#include <nerrno.h>	*/

/* #include <stdlib.h> */
#include <stdio.h>
#include <ctype.h>

#include "sostream.h"

/************************************************************************/
/*                                                                      */
/************************************************************************/
int SoStreamSetServer(char *service, unsigned short defport)
{
    int sock;
    unsigned short port;
    struct servent *sp;
    struct sockaddr_in server;
    char t_service[32];

    if((sock = SoStreamOpen()) == -1)
        return(-1);

    strcpy(t_service, service);
    if(isdigit(*t_service))
        port = (unsigned short)atoi(t_service);
    else if ((sp = getservbyname(t_service, "tcp")) != NULL)
        port = (unsigned short)ntohs((unsigned short)(sp->s_port));
    else {
        port = defport;
        fprintf(stderr, "Unknown service: %s, using port %u\n",
                t_service, port);
    }

    server.sin_family = AF_INET;
    server.sin_port = htons(port);
    server.sin_addr.s_addr = INADDR_ANY;
    /* (struct sockaddr *) */
    if (bind(sock,  (void *)&server, sizeof(server)) < 0) {
        perror("Binding server socket");
        SoStreamClose(sock);
        return (-1);
    }

    if (listen(sock, 5) != 0) {
        perror("Listening");
        SoStreamClose(sock);
        return (-1);
    }
    return(sock);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int SoStreamConnect(char *host, char *service, unsigned short defport)
{
    int sock;
    unsigned short port;
    unsigned long vaddr;
    struct sockaddr_in server;
    struct hostent *hostnm;
    struct servent *sp;

    memset(&server, 0, sizeof(server));
    server.sin_family = AF_INET;
    if ((vaddr = inet_addr(host)) != ((unsigned long) 0xffffffff))
        memcpy(&server.sin_addr.s_addr, &vaddr, sizeof(struct in_addr));
    else if ((hostnm = gethostbyname(host)) != NULL)
        server.sin_addr.s_addr = *((unsigned long *)(hostnm->h_addr_list[0]));
    else {
        fprintf(stderr, "%s: ", host);
        switch(errno) {
        case HOST_NOT_FOUND:
            fprintf(stderr, "Authoritative Answer Host not found\n");
            break;
        case TRY_AGAIN:
            fprintf(stderr, "Non-Authoritive Host not found, or SERVERFAIL\n");
            break;
        case NO_RECOVERY:
            fprintf(stderr, "Non recoverable errors, FORMERR, REFUSED, NOTIMP\n");
            break;
        case NO_DATA:
            fprintf(stderr, "Valid name, no data record of requested type\n");
            break;
        default:
            fprintf(stderr, "Unexpected error %d\n", errno);
            break;
        }
        return (-1);
    }

    if((sock = SoStreamOpen()) == -1)
        return (-1);

//    strcpy(service, service);
    if(isdigit(*service))
        port = (unsigned short)atoi(service);
    else if ((sp = getservbyname(service, "tcp")) != NULL)
        port = (unsigned short)ntohs((unsigned short)(sp->s_port));
    else {
        port = defport;
        fprintf(stderr, "Unknown service: %s, using port %u\n",
                service, port);
    }

    server.sin_port = htons(port);

    if (connect(sock, (void *) & server, sizeof(server)) < 0) {
        fprintf(stderr, "%s [%s] port %u", host,
                inet_ntoa(server.sin_addr), port);
        perror("");
        SoStreamClose(sock);
	return (-1);
    }
    return (sock);
}
